@@ -17,6 +17,14 @@ if TYPE_CHECKING:
         AppTransactionContext,
         use_app_type,
     )
+    from jararaca.observability.decorators import TracedClass, TracedFunc, traced_class
+    from jararaca.observability.hooks import (
+        add_event,
+        get_tracing_provider,
+        record_exception,
+        set_span_status,
+        spawn_trace,
+    )
     from jararaca.observability.interceptor import ObservabilityInterceptor
     from jararaca.observability.providers.otel import OtelObservabilityProvider
     from jararaca.persistence.sort_filter import (
@@ -222,6 +230,14 @@ if TYPE_CHECKING:
         "HttpPut",
         "HttpDelete",
         "ObservabilityInterceptor",
+        "TracedFunc",
+        "TracedClass",
+        "traced_class",
+        "spawn_trace",
+        "add_event",
+        "set_span_status",
+        "record_exception",
+        "get_tracing_provider",
         "QueryInjector",
         "HttpMicroservice",
         "use_current_container",
@@ -392,6 +408,14 @@ _dynamic_imports: "dict[str, tuple[str, str, str | None]]" = {
     "HttpPut": (__SPEC_PARENT__, "rpc.http.decorators", "Put"),
     "HttpDelete": (__SPEC_PARENT__, "rpc.http.decorators", "Delete"),
     "ObservabilityInterceptor": (__SPEC_PARENT__, "observability.interceptor", None),
+    "TracedFunc": (__SPEC_PARENT__, "observability.decorators", None),
+    "TracedClass": (__SPEC_PARENT__, "observability.decorators", None),
+    "traced_class": (__SPEC_PARENT__, "observability.decorators", None),
+    "spawn_trace": (__SPEC_PARENT__, "observability.hooks", None),
+    "add_event": (__SPEC_PARENT__, "observability.hooks", None),
+    "set_span_status": (__SPEC_PARENT__, "observability.hooks", None),
+    "record_exception": (__SPEC_PARENT__, "observability.hooks", None),
+    "get_tracing_provider": (__SPEC_PARENT__, "observability.hooks", None),
     "QueryInjector": (__SPEC_PARENT__, "persistence.utilities", None),
     "HttpMicroservice": (__SPEC_PARENT__, "presentation.http_microservice", None),
     "use_current_container": (__SPEC_PARENT__, "microservice", None),
