@@ -50,6 +50,7 @@ from jararaca.messagebus.decorators import (
     MessageHandlerData,
     ScheduleDispatchData,
 )
+from jararaca.messagebus.implicit_headers import provide_implicit_headers
 from jararaca.messagebus.message import Message, MessageOf
 from jararaca.microservice import (
     AppTransactionContext,
@@ -1130,6 +1131,8 @@ class ScheduledMessageHandlerCallback:
                 AppTransactionContext(
                     controller_member_reflect=scheduled_action.controller_member,
                     transaction_data=SchedulerTransactionData(
+                        task_name=scheduled_action.spec.name
+                        or scheduled_action.callable.__qualname__,
                         scheduled_to=datetime.now(UTC),
                         cron_expression=scheduled_action.spec.cron,
                         triggered_at=datetime.now(UTC),
@@ -1469,7 +1472,9 @@ class MessageHandlerCallback:
         incoming_message_spec = MessageHandler.get_message_incoming(handler)
         assert incoming_message_spec is not None
 
-        with provide_shutdown_state(self.consumer.shutdown_state):
+        with provide_implicit_headers(aio_pika_message.headers), provide_shutdown_state(
+            self.consumer.shutdown_state
+        ):
             async with self.consumer.uow_context_provider(
                 AppTransactionContext(
                     controller_member_reflect=handler_data.controller_member,
@@ -1523,6 +1528,7 @@ class MessageHandlerCallback:
                             logger.exception(
                                 f"Error processing message {message_id} on topic {routing_key}: {str(base_exc)}"
                             )
+                            raise
 
                         # Handle rejection with retry logic
                         if incoming_message_spec.requeue_on_exception:
@@ -1538,6 +1544,7 @@ class MessageHandlerCallback:
                             await self.handle_reject_message(
                                 aio_pika_message, requeue=False, exception=base_exc
                             )
+
                     else:
                         # Message processed successfully, log and clean up any retry state
                         message_id = aio_pika_message.message_id or str(uuid.uuid4())
